<?php

namespace App\Http\Controllers;

use App\Jobs\CompactStrAndVideoAndCoverImageAndFontAndColorJob;
use App\Jobs\CompactStrAndVideoAndCoverImageJob;
use App\Jobs\ProcessVideoJob;
use App\Models\User;
use App\Models\Video;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Storage;
use Illuminate\Support\Str;

class VideoController extends Controller
{
    public function videoUpload(Request $request)
    {
        $videoPath = $request->file('video')->store('videos');
        $video = Video::create([
            'title' => $request->file('video')->getClientOriginalName(),
            'path' => $videoPath
        ]);
        $this->processVideo($video);
        return response()->json($video);
    }


    protected function processVideo(Video $video): void
    {
        ProcessVideoJob::dispatch($video);
    }

    public function compactVideoWithQueueOld(Request $request)
    {
        $request->validate([
            'video' => ['required', 'mimes:mkv, mp4'],
            'srt' => ['required', 'mimes:srt'],
            'cover_image' => ['required', 'mimes:image'],
            'end_image' => ['required', 'mimes:image'],
            'font' => ['required', 'mimes'],
            'background_color' => ['required', 'string']
        ]);
//
        $folderPath = Storage::path('videos');
        $user = User::first();
        $files = Storage::files('videos');
        $videos = array_filter($files, fn($file) => str_ends_with($file, '.mp4'));
        if(!Storage::exists(storage_path('final-converted/'))){
            Storage::makeDirectory(('final-converted/'));
        }
        foreach($videos as $video) {
            $baseName = pathinfo($video, PATHINFO_FILENAME);
            $srtFile = $folderPath.'/'.$baseName.'.srt';
            $coverImage = $folderPath.'/cover_image.jpg';
            $endImage = $folderPath.'/end_image.jpg';
            $outputPath = storage_path('final-converted');

            CompactStrAndVideoAndCoverImageJob::dispatch(
                 storage_path('app/'.$video),
                 $srtFile,
                 $coverImage,
                 $endImage,
                $outputPath,
                $user
            );
        }
    }


    public function compactVideoWithQueue()
    {
        $folderPath = Storage::path('videos');
        $user = User::first();
        $files = Storage::files('videos');
        $videos = array_filter($files, fn($file) => str_ends_with($file, '.mp4'));

        if (!Storage::disk('local')->exists('final-converted')) {
            Storage::disk('local')->makeDirectory('final-converted');
        }

        foreach ($videos as $video) {
            $baseName = pathinfo($video, PATHINFO_FILENAME);
            $srtFile = $folderPath . '/' . $baseName . '.srt';
            $coverImage = $folderPath . "/cover_image_{$baseName}.jpg";
            $endImage = $folderPath . "/end_image_{$baseName}.jpg";
            $outputPath = Storage::disk('local')->path('final-converted');

            CompactStrAndVideoAndCoverImageJob::dispatch(
                Storage::disk('local')->path($video),
                $srtFile,
                $coverImage,
                $endImage,
                $outputPath,
                $user
            );
        }
    }

    public function convertVideo(Request $request)
    {
        $request->validate([
            'video'           => 'required|mimes:mp4,mov,avi|max:50000',
            'srt'             => 'required|file|mimes:srt,txt|max:10240',
            'cover_image'     => 'required|image|mimes:jpeg,png,jpg',
            'end_image'       => 'required|image|mimes:jpeg,png,jpg',
            'font'            => 'required|file|mimetypes:font/ttf,font/otf,font/sfnt,application/octet-stream',
            'background_color'=> 'required|string',
            'primary_color'   => ['required', 'string']
        ]);

        // ذخیره فایل‌ها در storage
        $videoPath = $request->file('video')->storeAs('', $request->file('video')->getClientOriginalName(), 'local');
        $srtPath = $request->file('srt')->storeAs('', $request->file('srt')->getClientOriginalName(), 'local');
        $coverImagePath = $request->file('cover_image')->storeAs('', $request->file('cover_image')->getClientOriginalName(), 'local');
        $endImagePath = $request->file('end_image')->storeAs('', $request->file('end_image')->getClientOriginalName(), 'local');
        $fontPath = $request->file('font')->storeAs('', $request->file('font')->getClientOriginalName(), 'local');
        $backgroundColor = $request->input('background_color');
        $primaryColor = $request->input('primary_color');
        if (!Storage::disk('local')->exists('processed')) {
            Storage::disk('local')->makeDirectory('processed');
        }

        $outputPath = Storage::disk('local')->path('processed');
        CompactStrAndVideoAndCoverImageAndFontAndColorJob::dispatch(
            storage_path('app/'.$videoPath),
            storage_path('app/'.$srtPath),
            storage_path('app/'.$coverImagePath),
            storage_path('app/'.$endImagePath),
            storage_path('app/'.$fontPath),
            $primaryColor,
            $backgroundColor,
            $outputPath,
        );

        return response()->json([
            'message' => 'Processing started...',
        ], 202);
    }


}
