<?php

namespace App\Jobs;

use Illuminate\Bus\Batch;
use Illuminate\Bus\Batchable;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Bus;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Log;
use Throwable;

class CompactStrAndVideoAndCoverImageAndFontAndColorJob implements ShouldQueue
{
    use Batchable, Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    private string $videoPath;
    private string $srtPath;
    private string $coverImagePath;
    private string $endImagePath;
    private string $fontPath;
    private string $primaryColor;
    private string $backgroundColor;
    private string $outputPath;



    public function __construct($video_path, $srt_path, $cover_image_path, $end_image_path, $font_path, $primary_color, $background_color, $output_path)
    {
        $this->videoPath = $video_path;
        $this->srtPath = $srt_path;
        $this->coverImagePath = $cover_image_path;
        $this->endImagePath = $end_image_path;
        $this->fontPath = $font_path;
        $this->primaryColor = $primary_color;
        $this->backgroundColor = $background_color;
        $this->outputPath = $output_path;
    }

    public function handle()
    {

        $qualities = [
            '720p'  => ['width' => 1280, 'height' => 720, 'bitrate' => '3M'],
            '1080p' => ['width' => 1920, 'height' => 1080, 'bitrate' => '5M'],
//            '1440p' => ['width' => 2560, 'height' => 1440, 'bitrate' => '8M'],
        ];

        $videoNameWithoutExt = pathinfo($this->videoPath, PATHINFO_FILENAME);
        $coverVideo = storage_path("app/temp_cover_{$videoNameWithoutExt}.mp4");
        $mainVideo = storage_path("app/temp_main_{$videoNameWithoutExt}.mp4");
        $endVideo = storage_path("app/temp_end_{$videoNameWithoutExt}.mp4");
        $concatList = storage_path("app/concat_{$videoNameWithoutExt}.txt");
        $finalVideo = storage_path("app/final_video_{$videoNameWithoutExt}.mp4");
        $finalOutput = storage_path("app/final_output_{$videoNameWithoutExt}.mp4");
        $audioFile = storage_path("app/main_audio_{$videoNameWithoutExt}.aac");
        $mergedAudio = storage_path("app/merged_audio_{$videoNameWithoutExt}.aac");
        $silentAudio = storage_path("app/silent_audio_{$videoNameWithoutExt}.aac");

        $convertedBackgroundColor = $this->convertToBGR($this->backgroundColor);
        $convertedPrimaryColor = $this->convertToBGR($this->primaryColor);
        $videFrameRate = shell_exec("ffprobe -v error -select_streams v:0 -show_entries stream=r_frame_rate -of csv=p=0 {$this->videoPath} | bc -l");
        $videFrameRate = intval($videFrameRate) > 0 ? intval($videFrameRate) : 30;

        $dimensions = trim(shell_exec("ffprobe -v error -select_streams v:0 -show_entries stream=width,height -of csv=p=0 {$this->videoPath}"));

        list($width, $height) = explode(',', $dimensions);

        shell_exec("ffmpeg -loop 1 -framerate {$videFrameRate} -t 5 -i {$this->coverImagePath} -vf scale={$width}:{$height} -c:v libx264 -pix_fmt yuv420p -an -y {$coverVideo}");

        shell_exec("ffmpeg -i {$this->videoPath} -vn -acodec copy -y {$audioFile}");

        shell_exec("ffmpeg -i {$this->videoPath} -vf \"subtitles={$this->srtPath}:force_style='FontName={$this->fontPath},PrimaryColour=&H{$convertedPrimaryColor},BackColour=&H{$convertedBackgroundColor},BorderStyle=3,Outline=1'\" -c:v libx264 -preset slow -crf 23 -c:a copy -y {$mainVideo}");

        shell_exec("ffmpeg -loop 1 -framerate {$videFrameRate} -t 3 -i {$this->endImagePath} -vf scale={$width}:{$height} -c:v libx264 -pix_fmt yuv420p -an -y {$endVideo}");

        shell_exec("ffmpeg -f lavfi -t 5 -i anullsrc=r=44100:cl=stereo -c:a aac -b:a 192k -y {$silentAudio}");

        shell_exec("ffmpeg -i {$silentAudio} -i {$audioFile} -filter_complex \"[0:a][1:a]concat=n=2:v=0:a=1[out]\" -map \"[out]\" -c:a aac -b:a 192k -y {$mergedAudio}");

        file_put_contents($concatList, "file '{$coverVideo}'\nfile '{$mainVideo}'\nfile '{$endVideo}'\n");
        shell_exec("ffmpeg -f concat -safe 0 -i {$concatList} -c:v libx264 -preset slow -crf 23 -an -y {$finalVideo}");

        shell_exec("ffmpeg -i {$finalVideo} -i {$mergedAudio} -c:v copy -c:a aac -b:a 192k -movflags +faststart -y {$finalOutput}");
        $jobs = [];
        $filesTemp = [$coverVideo, $mainVideo, $endVideo, $concatList, $finalVideo, $finalOutput, $audioFile, $mergedAudio, $silentAudio];

        foreach ($qualities as $quality => $config) {
            $jobs[] = new ConvertVideoForEachQuality($this->outputPath, $quality, $finalOutput, $config['width'], $config['height'], $config['bitrate'], $videoNameWithoutExt);
        }
        Bus::batch($jobs)
            ->name('video_conversion')
            ->finally(function() use($filesTemp){
                Log::alert('inside compact srt and ...');
                foreach($filesTemp as $file){
                    if(File::exists($file)){
                        File::delete($file);
                    }
                }
//                event(new VideoConversionProgress($batchId, 'completed', 100));

            })->catch(function (Throwable $e){
            })->dispatch();

    }

    function convertToBGR($hexColor) {
        // تبدیل #RRGGBB به BGR
        $hexColor = ltrim($hexColor, '#'); // حذف #
        if (strlen($hexColor) !== 6) {
            return '000000'; // مقدار پیش‌فرض مشکی در صورت مقدار نامعتبر
        }
        return substr($hexColor, 4, 2) . substr($hexColor, 2, 2) . substr($hexColor, 0, 2);
    }

}

