<?php

namespace App\Jobs;

use Illuminate\Bus\Batch;
use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Bus;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Str;
use Throwable;

class CompactStrAndVideoAndCoverImageJob implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    protected $videoPath;
    protected $srtPath;
    protected $coverImagePath;
    protected $endImagePath;
    protected $outputPath;

    public function __construct($videoPath, $srtPath, $coverImagePath, $endImagePath, $outputPath)
    {
        $this->videoPath = $videoPath;
        $this->srtPath = $srtPath;
        $this->coverImagePath = $coverImagePath;
        $this->endImagePath = $endImagePath;
        $this->outputPath = $outputPath;
    }

    public function handle()
    {

        $qualities = [
            '720p'  => ['width' => 1280, 'height' => 720, 'bitrate' => '3M'],
            '1080p' => ['width' => 1920, 'height' => 1080, 'bitrate' => '5M'],
//            '1440p' => ['width' => 2560, 'height' => 1440, 'bitrate' => '8M'],
        ];
        $videoNameWithoutExt = pathinfo($this->videoPath, PATHINFO_FILENAME);
        $coverVideo = storage_path("app/temp_cover_{$videoNameWithoutExt}.mp4");
        $mainVideo = storage_path("app/temp_main_{$videoNameWithoutExt}.mp4");
        $endVideo = storage_path("app/temp_end_{$videoNameWithoutExt}.mp4");
        $concatList = storage_path("app/concat_{$videoNameWithoutExt}.txt");
        $finalVideo = storage_path("app/final_video_{$videoNameWithoutExt}.mp4");
        $finalOutput = storage_path("app/final_output_{$videoNameWithoutExt}.mp4");
        $audioFile = storage_path("app/main_audio_{$videoNameWithoutExt}.aac");
        $mergedAudio = storage_path("app/merged_audio_{$videoNameWithoutExt}.aac");
        $silentAudio = storage_path("app/silent_audio_{$videoNameWithoutExt}.aac");


        $videFrameRate = shell_exec("ffprobe -v error -select_streams v:0 -show_entries stream=r_frame_rate -of csv=p=0 {$this->videoPath} | bc -l");
        $videFrameRate = intval($videFrameRate) > 0 ? intval($videFrameRate) : 30;

        $dimensions = trim(shell_exec("ffprobe -v error -select_streams v:0 -show_entries stream=width,height -of csv=p=0 {$this->videoPath}"));

        list($width, $height) = explode(',', $dimensions);

        shell_exec("ffmpeg -loop 1 -framerate {$videFrameRate} -t 5 -i {$this->coverImagePath} -vf scale={$width}:{$height} -c:v libx264 -pix_fmt yuv420p -an -y {$coverVideo}");

        shell_exec("ffmpeg -i {$this->videoPath} -vn -acodec copy -y {$audioFile}");

        shell_exec("ffmpeg -i {$this->videoPath} -vf \"subtitles={$this->srtPath}\" -c:v libx264 -preset slow -crf 23 -c:a copy -y {$mainVideo}");

        shell_exec("ffmpeg -loop 1 -framerate {$videFrameRate} -t 3 -i {$this->endImagePath} -vf scale={$width}:{$height} -c:v libx264 -pix_fmt yuv420p -an -y {$endVideo}");

        shell_exec("ffmpeg -f lavfi -t 5 -i anullsrc=r=44100:cl=stereo -c:a aac -b:a 192k -y {$silentAudio}");

        shell_exec("ffmpeg -i {$silentAudio} -i {$audioFile} -filter_complex \"[0:a][1:a]concat=n=2:v=0:a=1[out]\" -map \"[out]\" -c:a aac -b:a 192k -y {$mergedAudio}");

        file_put_contents($concatList, "file '{$coverVideo}'\nfile '{$mainVideo}'\nfile '{$endVideo}'\n");
        shell_exec("ffmpeg -f concat -safe 0 -i {$concatList} -c:v libx264 -preset slow -crf 23 -an -y {$finalVideo}");

        shell_exec("ffmpeg -i {$finalVideo} -i {$mergedAudio} -c:v copy -c:a aac -b:a 192k -movflags +faststart -y {$finalOutput}");
        $jobs = [];
        $filesTemp = [$coverVideo, $mainVideo, $endVideo, $concatList, $finalVideo, $finalOutput, $audioFile, $mergedAudio, $silentAudio];
        $batchId = Str::uuid();

        foreach ($qualities as $quality => $config) {
            $jobs[] = ConvertVideoForEachQuality::dispatch($this->outputPath, $quality, $finalOutput, $config['width'], $config['height'], $config['bitrate'], $videoNameWithoutExt, $batchId);
        }
        $batch = Bus::batch($jobs)
            ->name('video_conversion')
            ->finally(function() use($batchId, $filesTemp){
                foreach($filesTemp as $file){
                    if(File::exists($file)){
                        File::delete($file);
                    }
                }

            })->catch(function (Batch $batch, Throwable $e){

            })->dispatch();

    }

}

